MAX_PAGES_ALLOWED_BY_API = 20

APPLICATIONS = 'applications'
ASSETS = 'assets'
ASSET_GROUPS = 'asset_groups'
CONNECTORS = 'connectors'
CONNECTOR_RUNS = 'connector_runs'
VULNERABILITIES = 'vulnerabilities'
VULNERABILITY_DEFINITIONS = 'vulnerability-definitions'
FIXES = 'fixes'
USERS = 'users'
ROLES = 'roles'
DASHBOARD_GROUPS = 'dashboard_groups'
