IP_ADDRESS = "ip_address"
MAC_ADDRESS = "mac_address"
HOSTNAME = "hostname"
DATABASE = "database"
URL = "url"
NETBIOS = "netbios"
FQDN = "fqdn"
FILE = "file_search"
APPLICATION = "application"

LOCATOR_TYPES = [
    IP_ADDRESS,
    MAC_ADDRESS,
    HOSTNAME,
    DATABASE,
    URL,
    NETBIOS,
    FQDN,
    FILE,
    APPLICATION,
]
